package com.rjconsultores.ventaboletos.layouts;

import static com.rjconsultores.ventaboletos.utils.FormataUtil.formataDataToString;
import static com.rjconsultores.ventaboletos.utils.FormataUtil.formatarValorPorTamanho;
import static com.rjconsultores.ventaboletos.utils.FormataUtil.formatarValorPorTamanhoZeroEsquerda;

import java.util.Collection;
import java.util.List;

import org.jrimum.texgit.Record;

import com.rjconsultores.ventaboletos.blocos.CabecalhoRemessa;
import com.rjconsultores.ventaboletos.blocos.CabecalhoRetorno;
import com.rjconsultores.ventaboletos.blocos.DadosBoleto;
import com.rjconsultores.ventaboletos.blocos.DetalheObrigatorio;
import com.rjconsultores.ventaboletos.blocos.DetalheRetorno;
import com.rjconsultores.ventaboletos.blocos.RateioDeCredito;
import com.rjconsultores.ventaboletos.blocos.RodapeRemessa;
import com.rjconsultores.ventaboletos.blocos.RodapeRetorno;
import com.rjconsultores.ventaboletos.blocos.safra.CabecalhoRemessaSafra;
import com.rjconsultores.ventaboletos.blocos.safra.DetalheObrigatorioSafra;
import com.rjconsultores.ventaboletos.blocos.safra.RodapeRemessaSafra;
import com.rjconsultores.ventaboletos.enuns.BancoLayout;

public class SafraFlatFile extends AbstractFlatFile implements RemessaInterface {

	public SafraFlatFile() {
		super(BancoLayout.SAFRA_400_Envio);
	}
	
	@Override
	protected void createHeader(CabecalhoRemessa cabecalhoRemessaParam) {  
        
		Record header = getFlatFile().createRecord("Header");  
		
		CabecalhoRemessaSafra cabecalhoRemessa = (CabecalhoRemessaSafra) cabecalhoRemessaParam;	
	  
		header.setValue("NumAgencia", formatarValorPorTamanho(cabecalhoRemessa.getNumAgencia(), 5));         
        header.setValue("NumContaCorrente", formatarValorPorTamanho(cabecalhoRemessa.getNumContaCorrente(), 8));
        header.setValue("DigContaCorrente", formatarValorPorTamanho(cabecalhoRemessa.getDigContaCorrente(), 1));
        header.setValue("NomeEmpresa", formatarValorPorTamanho(cabecalhoRemessa.getNomeEmpresa(), 30));
        header.setValue("DataGravacao", formataDataToString(cabecalhoRemessa.getDataGravacao(), TAMANHO_DATA, DATA));    
        header.setValue(SEQUENCIAL_REMESSA, formatarValorPorTamanho(cabecalhoRemessa.getNumeroSequencialRemessa().toString(), 3));  
        header.setValue(SEQUENCIAL_REGISTRO, formatarValorPorTamanho(cabecalhoRemessa.getNumeroSequencialRegistro().toString(), 6)); 
        
        getFlatFile().addRecord(header);   
    } 
	
	@Override
	protected void createTransacaoTitulos(DetalheObrigatorio tituloParam) {  
        
		Record detalhe = getFlatFile().createRecord("TransacaoTitulo");  
		
		DetalheObrigatorioSafra titulo = (DetalheObrigatorioSafra) tituloParam;
		
		detalhe.setValue("CpfCnpj", formatarValorPorTamanho(titulo.getCpfCnpj().toString(), 2));
		detalhe.setValue("NumCpfCnpj", formatarValorPorTamanho(titulo.getNumCpfCnpj(), 14));
		detalhe.setValue("NumAgencia", formatarValorPorTamanho(titulo.getNumAgencia(), 5));         
		detalhe.setValue("NumContaCorrente", formatarValorPorTamanho(titulo.getNumContaCorrente(), 8));
		detalhe.setValue("DigContaCorrente", formatarValorPorTamanho(titulo.getDigContaCorrente(), 1));
		detalhe.setValue("NossoNumero", formatarValorPorTamanho(titulo.getNossoNumero(), 9));
		detalhe.setValue("IdentificacaoTitulo", formatarValorPorTamanho(titulo.getIdBoletoFechamento(), 10));
		detalhe.setValue("Vencimento", formataDataToString(titulo.getVencimento(), TAMANHO_DATA, DATA));
		detalhe.setValue("Valor", titulo.getValor());
		detalhe.setValue("NumAgenciaCobranca", formatarValorPorTamanho(titulo.getNumAgencia(), 5));
		detalhe.setValue("EspecieDeTitulo", formatarValorPorTamanho(titulo.getEspecieDeTitulo(), 2));
		detalhe.setValue("Aceite", titulo.getAceite().getValue());
		detalhe.setValue("Emissao", formataDataToString(titulo.getDataEmissao(), TAMANHO_DATA, DATA));
		detalhe.setValue("Instrucao1", formatarValorPorTamanho(titulo.getInstrucao1(), 2));
		detalhe.setValue("Instrucao2", formatarValorPorTamanho(titulo.getInstrucao2(), 2));
		detalhe.setValue("JurosDeMora", titulo.getJurosDeMora());
		detalhe.setValue("DataMulta", formataDataToString(titulo.getDataMulta(), TAMANHO_DATA, DATA));
		detalhe.setValue("PercentualMulta", titulo.getPercMulta() );		
		detalhe.setValue("DataLimiteDesconto", formataDataToString(titulo.getDataLimiteDesconto(), TAMANHO_DATA, DATA));
		detalhe.setValue("ValorDesconto", titulo.getValorDesconto());
		detalhe.setValue("ValorIOF", titulo.getValorIOF());
		detalhe.setValue("ValorAbatimentoConcedido", titulo.getValorAbatimentoConcedido());
		detalhe.setValue("TipoInscricaoPagador", formatarValorPorTamanho(titulo.getTipoInscricaoPagador().toString(), 2));
		detalhe.setValue("NumeroInscricaoPagador", formatarValorPorTamanho(titulo.getNumeroInscricaoPagador(), 14));
		detalhe.setValue("NomePagador", formatarValorPorTamanho(titulo.getNomePagador(), 40));
		detalhe.setValue("EnderecoPagador", formatarValorPorTamanho(titulo.getEnderecoPagador(), 30));
		detalhe.setValue("BairroPagador", formatarValorPorTamanho(titulo.getBairroPagador(), 10));
		detalhe.setValue("CepPagador", formatarValorPorTamanho(titulo.getCepPagador(), 8));
		detalhe.setValue("CidadePagador", formatarValorPorTamanho(titulo.getCidadePagador(), 15));
		detalhe.setValue("EstadoPagador", formatarValorPorTamanho(titulo.getEstadoPagador(), 2));
		detalhe.setValue("SacadorAvalista", formatarValorPorTamanho(titulo.getSacadorAvalista(), 30));
		detalhe.setValue(SEQUENCIAL_REMESSA, formatarValorPorTamanho(titulo.getNumeroSequencialRemessa().toString(), 3));
		detalhe.setValue(SEQUENCIAL_REGISTRO, formatarValorPorTamanho(titulo.getNumeroSequencialRegistro().toString(), 6));
	    
		getFlatFile().addRecord(detalhe); 
    } 
	
	@Override
	protected void createTrailler(RodapeRemessa trailler){
		
		Record trailer = getFlatFile().createRecord("Trailler");
		RodapeRemessaSafra rodape = (RodapeRemessaSafra) trailler;
		
        trailer.setValue("QtdTitulos", formatarValorPorTamanho(rodape.getQtdTitulos().toString(), 8));
        trailer.setValue("ValorTotal", formatarValorPorTamanhoZeroEsquerda(rodape.getValorTotalTitulos().toString().replace(".", ""), 15));
        trailer.setValue(SEQUENCIAL_REMESSA, formatarValorPorTamanho(rodape.getNumeroSequencialRemessa().toString(), 3));
        trailer.setValue(SEQUENCIAL_REGISTRO, formatarValorPorTamanho(rodape.getNumeroSequencialRegistro().toString(), 6));
        
        getFlatFile().addRecord(trailer); 
	}

	@Override
	protected CabecalhoRetorno readHeader(Record header) {
		throw new UnsupportedOperationException(SOMENTE_CLASSE_DE_RETORNO);
	}

	@Override
	protected List<DetalheRetorno> readTransacaoTitulos(Collection<Record> registrosDeTransacoes) {
		throw new UnsupportedOperationException(SOMENTE_CLASSE_DE_RETORNO);
	}

	@Override
	protected RodapeRetorno readTrailler(Record trailler) {
		throw new UnsupportedOperationException(SOMENTE_CLASSE_DE_RETORNO);
	}
	
	@Override
	protected Record createDadosBoletos(DadosBoleto boleto) {  
		throw new UnsupportedOperationException("Não necessário...");
	}
	
	@Override
	protected Record createRateioDeCredito(RateioDeCredito rateioDeCredito) {
		throw new UnsupportedOperationException("Não necessário...");
	}
}
